﻿;==============================================================================================================================================================================
; Write the error log to disk in the temporary directory
;==============================================================================================================================================================================

Procedure WriteErrorLog(ModuleName.s = #EmptyString, Message.s = #EmptyString)
  
  ;------------------------------------------------------------------------------------------------
  ; First time this has run for the current session so create a new log file, overwrite the old one
  ;------------------------------------------------------------------------------------------------
  
  If Program\FirstTime = #False And ListSize(ErrorList()) <> #NoItems
    
    ;----------------------------------------------------------------------------------------------
    ; Setup the formatted date value or the header
    ;----------------------------------------------------------------------------------------------
    
    DateVal = Date()
    
    CurrentDate.s = NameOfDay(DayOfWeek(DateVal))    + " "
    
    CurrentDate.s + Str(Day(DateVal))                + " "
    
    CurrentDate.s + NameOfMonth(Month(DateVal) - 1)  + " "
    
    CurrentDate.s + Str(Year(DateVal))
    
    ;----------------------------------------------------------------------------------------------
    ; Try to create the file now
    ;----------------------------------------------------------------------------------------------
    
    ErrorLogId.i = CreateFile(#PB_Any, Program\ErrorFile)
    
    ;----------------------------------------------------------------------------------------------
    ; If we got a file handle, write the header details to it
    ;----------------------------------------------------------------------------------------------
    
    If ErrorLogId.i
      
      WriteStringN(ErrorLogId.i, "----------------------------------------------------------------------")
      
      WriteStringN(ErrorLogId.i, "New " + #BaseName + #Version + " program run,  " + CurrentDate.s)
      
      WriteStringN(ErrorLogId.i, "----------------------------------------------------------------------")
      
      CloseFile(ErrorLogId.i)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  EndIf

  ;------------------------------------------------------------------------------------------------
  ; Errors are ready to be written to the file if list is full or routine is called at program end
  ;------------------------------------------------------------------------------------------------
  
  If ListSize(ErrorList()) > 50
    
    ;----------------------------------------------------------------------------------------------
    ; Try to open the previously created ErrorList log file
    ;----------------------------------------------------------------------------------------------
    
    ErrorLogId.i = OpenFile(#PB_Any, Program\ErrorFile)
    
    ;----------------------------------------------------------------------------------------------
    ; Now let's see if we got a file handle
    ;----------------------------------------------------------------------------------------------
    
    If ErrorLogId.i
      
      ;--------------------------------------------------------------------------------------------
      ; Seek to the end of the file to add entries
      ;--------------------------------------------------------------------------------------------
      
      FileSeek(ErrorLogId.i, Lof(ErrorLogId.i))
      
      ;--------------------------------------------------------------------------------------------
      ; Iterate through the list of errors
      ;--------------------------------------------------------------------------------------------
      
      ForEach ErrorList()
        
        WriteStringN(ErrorLogId.i, FormatField("Time:",     18) + ErrorList()\ErrorTime     + ", ")
        
        WriteString(ErrorLogId.i,  FormatField("Module:",   18) + ErrorList()\ErrorModule   + ", ")
        
        WriteString(ErrorLogId.i,  FormatField("Message:",  18) + ErrorList()\ErrorMessage  + ", ")
        
      Next
      
      ;--------------------------------------------------------------------------------------------
      ; Close the error log
      ;--------------------------------------------------------------------------------------------
      
      CloseFile(ErrorLogId.i)
      
      ;--------------------------------------------------------------------------------------------
      ; Clear the list to start again
      ;--------------------------------------------------------------------------------------------
      
      ClearList(ErrorList())
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ;
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    AddElement(ErrorList())
    
    ErrorList()\ErrorTime     = FormatDate("%hh:%ii:%ss", Date())
    
    ErrorList()\ErrorModule   = ModuleName.s
    
    ErrorList()\ErrorMessage  = Message.s
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ;
  ;------------------------------------------------------------------------------------------------
  
EndProcedure

; IDE Options = PureBasic 5.43 LTS (Windows - x86)
; CursorPosition = 85
; FirstLine = 94
; Folding = -
; EnableXP