﻿;==============================================================================================================================================================================
; Try to open the system database and create missing tables
;==============================================================================================================================================================================

Procedure OpenSystemDatabase()
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  Protected FileHandle.i, DatabaseUpdate.s
  
  ;------------------------------------------------------------------------------------------------
  ; Create the database if it doesn't already exist.
  ;------------------------------------------------------------------------------------------------
  
  FileHandle.i = OpenFile(#PB_Any, Program\DatabaseName)
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  If FileHandle.i
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    CloseFile(FileHandle.i)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    Program\DatabaseHandle = OpenDatabase(#PB_Any, Program\DatabaseName, #EmptyString, #EmptyString, #PB_Database_SQLite)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    If Program\DatabaseHandle <> #DatabaseNotOpen
      
      ;--------------------------------------------------------------------------------------------
      ; Turn on auto database vacuum
      ;--------------------------------------------------------------------------------------------
      
      If Not DatabaseUpdate(Program\DatabaseHandle, "PRAGMA auto_vacuum = on")
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      DatabaseUpdate.s = "CREATE TABLE IF NOT EXISTS Mystuff("
      
      DatabaseUpdate.s  + "Itemname TEXT, "
      
      DatabaseUpdate.s  + "Supplier TEXT, "
      
      DatabaseUpdate.s  + "Manufacturer TEXT, "
      
      DatabaseUpdate.s  + "Modelnumber TEXT, "
      
      DatabaseUpdate.s  + "Serialnumber TEXT, "
      
      DatabaseUpdate.s  + "Barcode TEXT, "
      
      DatabaseUpdate.s  + "Boughton TEXT, "
      
      DatabaseUpdate.s  + "Working TEXT, "
      
      DatabaseUpdate.s  + "Whopaid TEXT, "
      
      DatabaseUpdate.s  + "Ownername TEXT, "
      
      DatabaseUpdate.s  + "Location TEXT, "
      
      DatabaseUpdate.s  + "Condition TEXT, "
      
      DatabaseUpdate.s  + "Category TEXT, "
      
      DatabaseUpdate.s  + "Importance TEXT, "
      
      DatabaseUpdate.s  + "Loanedto TEXT, "
      
      DatabaseUpdate.s  + "Comments TEXT, "
      
      DatabaseUpdate.s  + "Itempicture TEXT, "
      
      DatabaseUpdate.s  + "Itemdocument TEXT, "
      
      DatabaseUpdate.s  + "Cost TEXT, "
      
      DatabaseUpdate.s  + "Tax TEXT, "
      
      DatabaseUpdate.s  + "Delivery TEXT, "
      
      DatabaseUpdate.s  + "Othercost TEXT, "
      
      DatabaseUpdate.s  + "Resale TEXT, "
      
      DatabaseUpdate.s  + "Paidby TEXT, "
      
      DatabaseUpdate.s  + "Warrantyperiod TEXT, "
      
      DatabaseUpdate.s  + "Warrantyword TEXT, "
      
      DatabaseUpdate.s  + "Warrantyactive TEXT, "
      
      DatabaseUpdate.s  + "Warrantyextra TEXT, "
      
      DatabaseUpdate.s  + "Insured TEXT, "
      
      DatabaseUpdate.s  + "Insuredamount TEXT, "
      
      DatabaseUpdate.s  + "Insuredon TEXT, "
      
      DatabaseUpdate.s  + "Insurername TEXT, "
      
      DatabaseUpdate.s  + "Policynumber TEXT, "
      
      DatabaseUpdate.s  + "Contactnumber TEXT, "
      
      DatabaseUpdate.s  + "Contactname TEXT, "
      
      DatabaseUpdate.s  + "Contactdepartment TEXT, "
      
      DatabaseUpdate.s  + "Deleted BOOLEAN, "
      
      DatabaseUpdate.s  + "Favourite BOOLEAN, "
      
      DatabaseUpdate.s  + "Locked BOOLEAN, "
      
      DatabaseUpdate.s  + "Mystuffid INTEGER PRIMARY KEY AUTOINCREMENT, "
      
      DatabaseUpdate.s  + "UNIQUE (Itemname, Supplier, Manufacturer, "
      
      DatabaseUpdate.s  + "Modelnumber, Serialnumber, Barcode, "
      
      DatabaseUpdate.s  + "Boughton, Working, Whopaid, Ownername, "
      
      DatabaseUpdate.s  + "Location, Condition, Category, Comments) "
      
      DatabaseUpdate.s  + "ON CONFLICT FAIL)"
      
      If Not DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s)
        
        Debug DatabaseError()
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      DatabaseUpdate.s = "CREATE INDEX IF NOT EXISTS Mystuffindex ON Mystuff("
      
      DatabaseUpdate.s + "Itemname)"
      
      DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      DatabaseUpdate.s = "CREATE TABLE IF NOT EXISTS Locations("
      
      DatabaseUpdate.s + "Location TEXT, "
      
      DatabaseUpdate.s + "Iconname TEXT, "
      
      DatabaseUpdate.s + "Itemcount TEXT, "
      
      DatabaseUpdate.s + "Locationid INTEGER PRIMARY KEY AUTOINCREMENT, "
      
      DatabaseUpdate.s + "UNIQUE(Location))"
      
      DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      DatabaseUpdate.s = "CREATE TABLE IF NOT EXISTS Options("
      
      DatabaseUpdate.s + "Optionid INTEGER PRIMARY KEY AUTOINCREMENT, "
      
      DatabaseUpdate.s + "Snapformtoborders INTEGER, "    ; Snapformtoborders
      
      DatabaseUpdate.s + "Showtitlebarclock INTEGER, "    ; Showtitlebarclock
      
      DatabaseUpdate.s + "Tooltipsonoroff INTEGER, "      ; Tooltipsonoroff
      
      DatabaseUpdate.s + "Stickwindowontop INTEGER, "     ; Stickwindowontop
      
      DatabaseUpdate.s + "Colourbandinlist INTEGER, "     ; Colourbandinlist
      
      DatabaseUpdate.s + "Messageaftersave INTEGER)"      ; Messageaftersave
      
      If DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s) <> #DatabaseUpdateFail
        
        If CountRecords("Optionid", "Options") = "0"
          
          DatabaseUpdate.s = "INSERT INTO Options("
          
          DatabaseUpdate.s + "Snapformtoborders, "
          
          DatabaseUpdate.s + "Showtitlebarclock, "
          
          DatabaseUpdate.s + "Tooltipsonoroff, "
          
          DatabaseUpdate.s + "Stickwindowontop, "
          
          DatabaseUpdate.s + "Colourbandinlist, "
          
          DatabaseUpdate.s + "Messageaftersave) "
          
          DatabaseUpdate.s + "VALUES("
          
          DatabaseUpdate.s + "1, "                ; Snapformtoborders
          
          DatabaseUpdate.s + "1, "                ; Showtitlebarclock
          
          DatabaseUpdate.s + "1, "                ; Tooltipsonoroff
          
          DatabaseUpdate.s + "1, "                ; Stickwindowontop
          
          DatabaseUpdate.s + "1, "                ; Colourbandinlist
          
          DatabaseUpdate.s + "1)"                 ; Messageaftersave
          
          If DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s) <> #DatabaseUpdateFail
            
          Else
            
            MessageRequester("Table update error", "Could not add program option data to options table.", #PB_MessageRequester_Ok)
            
          EndIf
          
        EndIf
        
      Else
        
        MessageRequester("Table creation error", "Could not create option data table in Mystuff database.", #PB_MessageRequester_Ok)
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
    Else
      
      MessageRequester("Database open error", "Could not open Mystuff database file in database mode.", #PB_MessageRequester_Ok)
      
    EndIf
    
    ;------------------------------------------------------------------------------------------------
    ; 
    ;------------------------------------------------------------------------------------------------
    
  Else

    MessageRequester("Database open error", "Could not open or create the raw Mystuff database file on disk.", #PB_MessageRequester_Ok)

  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure


; IDE Options = PureBasic 5.43 LTS (Windows - x86)
; CursorPosition = 170
; FirstLine = 161
; Folding = -
; EnableAsm
; EnableUnicode
; EnableXP