﻿;==========================================================================================================================================================
; Show a list of all the keyboard shortcuts
;==========================================================================================================================================================

Procedure Keyboardshortcuts()
  
  ;----------------------------------------------------------------------------------------------
  ; Check if a previous window is open
  ;----------------------------------------------------------------------------------------------
  
  If Program\WindowMutex <> #True
    
    ;--------------------------------------------------------------------------------------------
    ; Try to open the data window
    ;--------------------------------------------------------------------------------------------
    
    If Window_Keyboardshortcuts()
      
      ;------------------------------------------------------------------------------------------
      ; Prevent user input on the main window
      ;------------------------------------------------------------------------------------------
      
      DisableWindow(#Window_Mystuff, #True)
      
      ;------------------------------------------------------------------------------------------
      ; Warn other components a window is open and not to attempt to open another
      ;------------------------------------------------------------------------------------------
      
      Program\WindowMutex = #True
      
      ;------------------------------------------------------------------------------------------
      ; Add the various keyboard shortcuts
      ;------------------------------------------------------------------------------------------
      
      AddKeyboardShortcut(#Window_Keyboardshortcuts, #Keys_Keyboardshortcuts_Escape, #Shortcut_Keyboardshortcuts_Escape)
      
      ;------------------------------------------------------------------------------------------
      ; Set the common window and statusbar message
      ;------------------------------------------------------------------------------------------
      
      Message.s = "Show " + #Program + " keyboard shortcuts"
      
      ;------------------------------------------------------------------------------------------
      ; Use the common message as a window identifier
      ;------------------------------------------------------------------------------------------
      
      Program\LastWindow = Message.s
      
      ;------------------------------------------------------------------------------------------
      ; Set the window title with the common message and next record number
      ;------------------------------------------------------------------------------------------
      
      SetWindowTitle(#Window_Keyboardshortcuts, Message.s)
      
      ;--------------------------------------------------------------------------------------------
      ; Remove borders from the details pane by remove windows theming support, gets rid of 1 pixel border
      ;--------------------------------------------------------------------------------------------
      
      RemoveBorders(#Gadget_Keyboardshortcuts_Keylist)
      
      ;------------------------------------------------------------------------------------------------------------------------------------------------------------------------
      ; Setup some qhuckie display macros for headings to avoid repeated typing
      ;------------------------------------------------------------------------------------------------------------------------------------------------------------------------
      
      Macro AddHeading(FirstString, SecondString)
        
        AddGadgetItem(#Gadget_Keyboardshortcuts_Keylist, #AtTheEndOfTheList, #EmptyString  + #LF$ + #EmptyString)

        AddGadgetItem(#Gadget_Keyboardshortcuts_Keylist, #AtTheEndOfTheList, FirstString   + #LF$ + SecondString)
        
        AddGadgetItem(#Gadget_Keyboardshortcuts_Keylist, #AtTheEndOfTheList, #EmptyString  + #LF$ + #EmptyString)
        
      EndMacro
      
      ;------------------------------------------------------------------------------------------------------------------------------------------------------------------------
      ; Setup some qhuckie display macros for content to avoid repeated typing
      ;------------------------------------------------------------------------------------------------------------------------------------------------------------------------
      
      Macro AddEntry(Shortcut, Keytext, Imagenumber)
        
        AddGadgetItem(#Gadget_Keyboardshortcuts_Keylist, #AtTheEndOfTheList, GetShortCutText(Shortcut) + #LF$ + Keytext, ImageID(Imagenumber))
        
      EndMacro
      
      ;------------------------------------------------------------------------------------------------------------------------------------------------------------------------
      ; File menu, main window
      ;------------------------------------------------------------------------------------------------------------------------------------------------------------------------
      
      AddHeading(#EmptyString, "File menu, main window")
      
      AddEntry(#Keys_Mystuff_Backupmystuff,             #Keystext_Mystuff_Backupmystuff,          #MenuBarIcon_Mystuff_Backupmystuff)
      
      AddEntry(#Keys_Mystuff_Deleteallmystuff,          #Keystext_Mystuff_Deleteallmystuff,       #MenuBarIcon_Mystuff_Deleteallmystuff)
      
      AddEntry(#Keys_Mystuff_Vacuummystuff,             #Keystext_Mystuff_Vacuummystuff,          #MenuBarIcon_Mystuff_Vacuummystuff)
      
      AddEntry(#Keys_Mystuff_Exportmystuff,             #Keystext_Mystuff_Exportmystuff,          #MenuBarIcon_Mystuff_Exportmystuff)
      
      AddEntry(#Keys_Mystuff_Importmystuff,             #Keystext_Mystuff_Importmystuff,          #MenuBarIcon_Mystuff_Importmystuff)
      
      AddEntry(#Keys_Mystuff_Exitmystuff,               #Keystext_Mystuff_Exitmystuff,            #MenuBarIcon_Mystuff_Exitmystuff)

      ;------------------------------------------------------------------------------------------
      ; Category menu, main window
      ;------------------------------------------------------------------------------------------
      
      AddHeading(#EmptyString, "Locations menu, main window")
      
      AddEntry(#Keys_Mystuff_Addlocation,               #Keystext_Mystuff_Addlocation,            #MenuBarIcon_Mystuff_Addlocation)
      
      AddEntry(#Keys_Mystuff_Copylocation,              #Keystext_Mystuff_Copylocation,           #MenuBarIcon_Mystuff_Copylocation)
      
      AddEntry(#Keys_Mystuff_Deletelocation,            #Keystext_Mystuff_Deletelocation,         #MenuBarIcon_Mystuff_Deletelocation)
      
      AddEntry(#Keys_Mystuff_Editlocation,              #Keystext_Mystuff_Editlocation,           #MenuBarIcon_Mystuff_Editlocation)
      
      AddEntry(#Keys_Mystuff_Exportlocations,           #Keystext_Mystuff_Exportlocations,        #MenuBarIcon_Mystuff_Exportlocations)
      
      AddEntry(#Keys_Mystuff_Importlocations,           #Keystext_Mystuff_Importlocations,        #MenuBarIcon_Mystuff_Importlocations)

      ;------------------------------------------------------------------------------------------
      ; Item menu, main window
      ;------------------------------------------------------------------------------------------
      
      AddHeading(#EmptyString, "Data menu, main window")
      
      AddEntry(#Keys_Mystuff_Addnewstuff,               #Keystext_Mystuff_Addnewstuff,            #MenuBarIcon_Mystuff_Addnewstuff)
      
      AddEntry(#Keys_Mystuff_Copyoldstuff,              #Keystext_Mystuff_Copyoldstuff,           #MenuBarIcon_Mystuff_Copyoldstuff)
      
      AddEntry(#Keys_Mystuff_Deleteoldstuff,            #Keystext_Mystuff_Deleteoldstuff,         #MenuBarIcon_Mystuff_Deleteoldstuff)
      
      AddEntry(#Keys_Mystuff_Editoldstuff,              #Keystext_Mystuff_Editoldstuff,           #MenuBarIcon_Mystuff_Editoldstuff)
      
      AddEntry(#Keys_Mystuff_Killthisstuff,             #Keystext_Mystuff_Killthisstuff,          #MenuBarIcon_Mystuff_Killthisstuff)
      
      AddEntry(#Keys_Mystuff_Lockthisstuff,             #Keystext_Mystuff_Lockthisstuff,          #MenuBarIcon_Mystuff_Lockthisstuff)
      
      AddEntry(#Keys_Mystuff_Favouritestuff,            #Keystext_Mystuff_Favouritestuff,         #MenuBarIcon_Mystuff_Favouritestuff)
      
      AddEntry(#Keys_Mystuff_Printthisstuff,            #Keystext_Mystuff_Printthisstuff,         #MenuBarIcon_Mystuff_Printthisstuff)
      
      AddEntry(#Keys_Mystuff_Pictureofstuff,            #Keystext_Mystuff_Pictureofstuff,         #MenuBarIcon_Mystuff_Pictureofstuff)

      ;------------------------------------------------------------------------------------------
      ; Utilities menu, main window
      ;------------------------------------------------------------------------------------------
      
      AddHeading(#EmptyString, "Search menu, main window")
      
      AddEntry(#Keys_Mystuff_CheckReturn,               #Keystext_Mystuff_CheckReturn,            #MenuBarIcon_Mystuff_Blank)

      ;------------------------------------------------------------------------------------------
      ; Options menu, main window
      ;------------------------------------------------------------------------------------------
      
      AddHeading(#EmptyString, "Options menu, main window")
      
      AddEntry(#Keys_Mystuff_Colourbandinlist,          #Keystext_Mystuff_Colourbandinlist,       #MenuBarIcon_Mystuff_Blank)
      
      AddEntry(#Keys_Mystuff_Lockmystuffwindow,         #Keystext_Mystuff_Lockmystuffwindow,      #MenuBarIcon_Mystuff_Blank)
      
      AddEntry(#Keys_Mystuff_Messageaftersave,          #Keystext_Mystuff_Messageaftersave,       #MenuBarIcon_Mystuff_Blank)
      
      AddEntry(#Keys_Mystuff_Stickwindowontop,          #Keystext_Mystuff_Stickwindowontop,       #MenuBarIcon_Mystuff_Blank)
      
      AddEntry(#Keys_Mystuff_Showtitlebarclock,         #Keystext_Mystuff_Showtitlebarclock,      #MenuBarIcon_Mystuff_Blank)
      
      AddEntry(#Keys_Mystuff_Snapformtoborders,         #Keystext_Mystuff_Snapformtoborders,      #MenuBarIcon_Mystuff_Blank)
      
      AddEntry(#Keys_Mystuff_Tooltipsonoroff,           #Keystext_Mystuff_Tooltipsonoroff,        #MenuBarIcon_Mystuff_Blank)
      
      AddEntry(#Keys_Mystuff_Setupmystuff,              #Keystext_Mystuff_Setupmystuff,           #MenuBarIcon_Mystuff_Blank)

      ;------------------------------------------------------------------------------------------
      ; Options menu, main window
      ;------------------------------------------------------------------------------------------
      
      AddHeading(#EmptyString, "Help menu, main window")
      
      AddEntry(#Keys_Mystuff_Aboutmystuff,              #Keystext_Mystuff_Aboutmystuff,           #MenuBarIcon_Mystuff_Aboutmystuff)
      
      AddEntry(#Keys_Mystuff_Keyboardshortcuts,         #Keystext_Mystuff_Keyboardshortcuts,      #MenuBarIcon_Mystuff_Keyboardshortcuts)
      
      AddEntry(#Keys_Mystuff_Showhelp,                  #Keystext_Mystuff_Showhelp,               #MenuBarIcon_Mystuff_Showhelp)

      ;------------------------------------------------------------------------------------------
      ; Options menu, main window
      ;------------------------------------------------------------------------------------------
      
      AddHeading(#EmptyString, "Miscellaneous menu, main window")

      AddEntry(#Keys_Mystuff_Statisticsformystuff,      #Keystext_Mystuff_Statisticsformystuff,   #MenuBarIcon_Mystuff_Statisticsformystuff)
      
      AddEntry(#Keys_Mystuff_Viewmystufflogfile,        #Keystext_Mystuff_Viewmystufflogfile,     #MenuBarIcon_Mystuff_Viewmystufflogfile)
      
      ;------------------------------------------------------------------------------------------
      ; Options menu, main window
      ;------------------------------------------------------------------------------------------
      
      AddHeading(#EmptyString, "Data sheet, Data window")

      AddEntry(#Keys_Datasheet_Menu_Addpicture,         #Keystext_Datasheet_Menu_Addpicture,      #MenuBarIcon_Mystuff_Addnewstuff)
      
      AddEntry(#Keys_Datasheet_Menu_Clearpicture,       #Keystext_Datasheet_Menu_Clearpicture,    #MenuBarIcon_Mystuff_Clearmystuff)
      
      AddEntry(#Keys_Datasheet_Menu_Deletepicture,      #Keystext_Datasheet_Menu_Deletepicture,   #MenuBarIcon_Mystuff_Deleteoldstuff)
      
      AddEntry(#Keys_Datasheet_Savemystuff,             #Keystext_Datasheet_Savemystuff,          #MenuBarIcon_Mystuff_Savemystuff)

      ;------------------------------------------------------------------------------------------
      ; Set the current operation details on the the main window status bar
      ;------------------------------------------------------------------------------------------
      
      SetInfoBarArea("Headings",  "Info",  Message.s, #PB_Compiler_Procedure)
      
      ;------------------------------------------------------------------------------------------
      ; Set focus on the appointment name field
      ;------------------------------------------------------------------------------------------
      
      SetActiveGadget(#Gadget_Keyboardshortcuts_Keylist)
      
      ;------------------------------------------------------------------------------------------
      ; Display status to the user
      ;------------------------------------------------------------------------------------------
      
    Else
      
      SetInfoBarArea("Headings",  "Error",  "Could not open the [" + Program\LastWindow + "] window.", #PB_Compiler_Procedure)
      
    EndIf
    
    ;--------------------------------------------------------------------------------------------
    ; Display status to the user
    ;--------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Warn",  "The [" +  Program\LastWindow + "] window is still open, close that one first.", #PB_Compiler_Procedure)
    
  EndIf
  
  ;----------------------------------------------------------------------------------------------
  ; 
  ;----------------------------------------------------------------------------------------------
  
EndProcedure

; IDE Options = PureBasic 5.43 LTS (Windows - x86)
; CursorPosition = 100
; FirstLine = 74
; Folding = -
; EnableAsm
; EnableUnicode
; EnableXP
; HideErrorLog