;==========================================================================================================================================================
; Show statistics about the current database
;==========================================================================================================================================================

Procedure Statisticsformystuff()
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  Macro ColourLine(LineNumber, Colour, Column = -1)
    
    SetGadgetItemColor(#Gadget_Statisticsformystuff_Statistics, LineNumber, #PB_Gadget_FrontColor, Colour, Column)
    
  EndMacro
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  Macro AddItem(AddString)
    
    AddGadgetItem(#Gadget_Statisticsformystuff_Statistics, #AtTheEndOfTheList, AddString)
    
  EndMacro
  
  ;------------------------------------------------------------------------------------------------
  ; Check if an existing window is opened
  ;------------------------------------------------------------------------------------------------
  
  If Program\WindowMutex <> #True

    ;----------------------------------------------------------------------------------------------
    ; Try to open the data form window
    ;----------------------------------------------------------------------------------------------

    If Window_Statisticsformystuff()
      
      ;--------------------------------------------------------------------------------------------
      ; Prevent user input on the main window
      ;--------------------------------------------------------------------------------------------
      
      DisableWindow(#Window_Mystuff, #True)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      Program\WindowMutex = #True
      
      ;--------------------------------------------------------------------------------------------
      ; Add this window's fast escape shortcut
      ;--------------------------------------------------------------------------------------------
      
      AddKeyboardShortcut(#Window_Statisticsformystuff, #Keys_Statisticsformystuff_Escape, #ShortCut_Statisticsformystuff_escape)
      
      ;--------------------------------------------------------------------------------------------
      ; Setup the window tool tips
      ;--------------------------------------------------------------------------------------------
      
      ;SelectWindowToolTips(#Window_Statisticsformystuff)

      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      SetWindowTitle(#Window_Statisticsformystuff, "Quick database statistics for my stuff")
      
      ;--------------------------------------------------------------------------------------------
      ; Remove borders from ListIconGadgets
      ;--------------------------------------------------------------------------------------------
      
      OldStyle = GetWindowLongPtr_(GadgetID(#Gadget_Statisticsformystuff_Statistics), #GWL_EXSTYLE)
      
      NewStyle = OldStyle & ( ~ #WS_EX_CLIENTEDGE)
      
      SetWindowLongPtr_(GadgetID(#Gadget_Statisticsformystuff_Statistics), #GWL_EXSTYLE, #Gadget_Statisticsformystuff_Statistics)
      
      SetWindowPos_(GadgetID(#Gadget_Statisticsformystuff_Statistics), 0, 0, 0, 0, 0, #SWP_SHOWWINDOW | #SWP_NOSIZE | #SWP_NOMOVE | #SWP_FRAMECHANGED)

      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      AddItem("Record information"          + #LF$ + #EmptyString)
      
      ColourLine(0, $F10A0E)
      
      AddItem(#EmptyString + #LF$ + #EmptyString)
      
      AddItem("Normal records"              + #LF$ + CountRecords("Mystuffid", "Mystuff", " WHERE Locked = '0' AND Deleted = '0'"))
      
      ColourLine(2, $1E1C98, 0)   : ColourLine(2, $922D6D, 1) 
      
      AddItem("Favourite records"           + #LF$ + CountRecords("Mystuffid", "Mystuff", " WHERE Favourite = '1'"))
      
      ColourLine(3, $1E1C98, 0)   : ColourLine(3, $922D6D, 1) 
      
      AddItem("Locked records"              + #LF$ + CountRecords("Mystuffid", "Mystuff", " WHERE Locked = '1'"))
      
      ColourLine(4, $1E1C98, 0)   : ColourLine(3, $922D6D, 1) 
      
      AddItem("Deleted records"             + #LF$ + CountRecords("Mystuffid", "Mystuff", " WHERE Deleted = '1'"))
      
      ColourLine(5, $1E1C98, 0)   : ColourLine(4, $922D6D, 1) 
      
      AddItem(#EmptyString                  + #LF$ + #EmptyString)
      
      AddItem("Access details"              + #LF$ + #EmptyString)
      
      ColourLine(7, $F10A0E)
      
      AddItem(#EmptyString                  + #LF$ + #EmptyString)
      
      AddItem("Created on"                  + #LF$ + FormatDate("%dd/%mm/%yyyy", GetFileDate(Program\DatabaseName, #PB_Date_Created)))
      
      ColourLine(9, $1E1C98, 0)   : ColourLine(8, $922D6D, 1) 
      
      AddItem("Last accessed"               + #LF$ + FormatDate("%dd/%mm/%yyyy", GetFileDate(Program\DatabaseName, #PB_Date_Accessed)))
      
      ColourLine(10, $1E1C98, 0)  : ColourLine(9, $922D6D, 1) 
      
      AddItem("Last modified"               + #LF$ + FormatDate("%dd/%mm/%yyyy", GetFileDate(Program\DatabaseName, #PB_Date_Modified)))
      
      ColourLine(11, $1E1C98, 0)  : ColourLine(10, $922D6D, 1) 
      
      AddItem(#EmptyString                  + #LF$ + #EmptyString)
      
      AddItem("Database file info"          + #LF$ + #EmptyString)
      
      ColourLine(13, $F10A0E)
      
      AddItem(#EmptyString                  + #LF$ + #EmptyString)
      
      AddItem("Database size"               + #LF$ + ByteCalc(FileSize(Program\DatabaseName)))
      
      ColourLine(15, $1E1C98, 0)  : ColourLine(14, $922D6D, 1) 
      
      AddItem("Database type"               + #LF$ + "SQLite")
      
      ColourLine(16, $1E1C98, 0)  : ColourLine(15, $922D6D, 1) 
      
      If DatabaseQuery(Program\DatabaseHandle, "Select sqlite_version()")
        
        While NextDatabaseRow(Program\DatabaseHandle)
          
          AddItem("SQLite driver version"   + #LF$ + GetDatabaseString(Program\DatabaseHandle, 0))
          
        Wend
        
        FinishDatabaseQuery(Program\DatabaseHandle)
        
      EndIf
      
      ColourLine(17, $1E1C98, 0)  : ColourLine(16, $922D6D, 1) 
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      SetInfoBarArea("Headings", "Error", "Could not open quick database statistics window. Program error", #PB_Compiler_Procedure)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Info", "Another program window is still open, close that one first.", #PB_Compiler_Procedure)
    
  EndIf

  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure


; IDE Options = PureBasic 5.43 LTS (Windows - x86)
; CursorPosition = 162
; FirstLine = 146
; Folding = -
; EnableXP