﻿;==========================================================================================================================================================
; Add a new location to the location bar
;==========================================================================================================================================================

Procedure Addlocation()
  
  ;------------------------------------------------------------------------------------------------
  ; Check if another window isn't open first
  ;------------------------------------------------------------------------------------------------
  
  If Program\WindowMutex <> #True
    
    ;----------------------------------------------------------------------------------------------
    ; Check to see if the Location window could be opened
    ;----------------------------------------------------------------------------------------------
    
    If Window_Location()  <> #NoWindowId
      
      ;--------------------------------------------------------------------------------------------
      ; If there are no other open windows, set the window open flag
      ;--------------------------------------------------------------------------------------------
      
      Program\WindowMutex = #True
      
      ;--------------------------------------------------------------------------------------------
      ; Add this window's fast escape shortcut
      ;--------------------------------------------------------------------------------------------
      
      AddKeyboardShortcut(#Window_Location, #Keys_Location_Escape,  #Shortcut_Location_Escape)
      
      AddKeyboardShortcut(#Window_Location, #Keys_Location_Save,    #Shortcut_Location_Save)
      
      AddKeyboardShortcut(#Window_Location, #Keys_Location_Exit,    #Shortcut_Location_Exit)
      
      ;--------------------------------------------------------------------------------------------
      ; Add the hoverable controls
      ;--------------------------------------------------------------------------------------------
      
      AddHoverGadget(GadgetID(#Gadget_Location_Save))
      
      AddHoverGadget(GadgetID(#Gadget_Location_Exit))
      
      ;--------------------------------------------------------------------------------------------
      ; These are your balloon tooltips used on the main form. All headings, balloon text and 
      ; icons are re-useable variables taken from the constants file.
      ;--------------------------------------------------------------------------------------------
      
      If Program\BalloonFontId
        
        BalloonToolTip(#Window_Location, #Gadget_Location_Save, Program\BalloonFontId, $FFFFFF, #Keystext_Location_Save,    #CRLF$ + GetShortCutText(#Keys_Location_Save) + #CRLF$ + #CRLF$ + WordWrapString(#LongKeysText_Location_Save, 30), GetIconHandle(#Image_Location_Save))
        
        BalloonToolTip(#Window_Location, #Gadget_Location_Exit, Program\BalloonFontId, $FFFFFF, #Keystext_Location_Exit,    #CRLF$ + GetShortCutText(#Keys_Location_Exit) + #CRLF$ + #CRLF$ + WordWrapString(#LongKeysText_Location_Exit, 30), GetIconHandle(#Image_Location_Exit))
        
      Else
        
        MessageRequester("Balloon font load error", "Could not load the ballon font so no tooltips will be loaded", #PB_MessageRequester_Ok)
        
      EndIf            
      
      ;--------------------------------------------------------------------------------------------
      ; Disable all input on the main window until this one is closed
      ;--------------------------------------------------------------------------------------------
      
      DisableWindow(#Window_Mystuff, #True)
      
      ;--------------------------------------------------------------------------------------------
      ; Hide the main window until we are finished with the data window
      ;--------------------------------------------------------------------------------------------
      
      HideWindow(#Window_Mystuff, #True)
      
      ;--------------------------------------------------------------------------------------------
      ; Add the window title for various procedures and visuals
      ;--------------------------------------------------------------------------------------------
      
      Program\LastWindow = "Add New Location"
      
      ;--------------------------------------------------------------------------------------------
      ; Load the user chosen icons to be selected for locations
      ;--------------------------------------------------------------------------------------------
      
      DirectoryId.i = ExamineDirectory(#PB_Any, Program\IconsDirectory, "*.ico")
      
      If DirectoryId.i
        
        Repeat
          
          TypeOfFile.i  = NextDirectoryEntry(DirectoryId.i) 
          
          IconName.s    = DirectoryEntryName(DirectoryId.i)
          
          If TypeOfFile.i = #True And IconName.s <> "." And IconName.s <> ".."
            
            If FileSize(Program\IconsDirectory  + IconName.s) <> #FileDoesNotExist
              
              IconHandle.i = LoadImage(#PB_Any, Program\IconsDirectory  + IconName.s)
              
              If IconHandle.i <>  #NoPictureLoaded
                
                AddGadgetItem(#Gadget_Location_Locationicon, #AtTheEndOfTheList, IconName.s, ImageID(IconHandle.i))
                
              Else
                
                AddGadgetItem(#Gadget_Location_Locationicon, #AtTheEndOfTheList, IconName.s, ImageID(#MenuBarIcon_Mystuff_Blank))
                
                SetInfoBarArea("Headings", "Error", "Could load the icon file as it is damaged.", #PB_Compiler_Procedure)
                
              EndIf
              
            Else
              
              AddGadgetItem(#Gadget_Location_Locationicon, #AtTheEndOfTheList, IconName.s, ImageID(#MenuBarIcon_Mystuff_Blank))
              
              SetInfoBarArea("Headings", "Error", "Icon file does not exist or is damaged.", #PB_Compiler_Procedure)
              
            EndIf
            
          EndIf 
          
        Until TypeOfFile.i = #False
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Setup the next possible record number variable
      ;--------------------------------------------------------------------------------------------
      
      NextRecord.s = Str(Val(LastRecord("Locationid", "Locations")) + 1)
      
      ;--------------------------------------------------------------------------------------------
      ; Setup the nextrecord generic string message
      ;--------------------------------------------------------------------------------------------
      
      NextRecordString.s  = " - Record: " + NextRecord.s + "?"
      
      ;--------------------------------------------------------------------------------------------
      ; Add a generic message for info bars and title areas
      ;--------------------------------------------------------------------------------------------
      
      Message.s = "Add new location"
      
      ;--------------------------------------------------------------------------------------------
      ; Set the window title bar with the record number and message
      ;--------------------------------------------------------------------------------------------
      
      SetWindowTitle(#Window_Location, Message.s + NextRecordString.s)
      
      ;--------------------------------------------------------------------------------------------
      ; Make sure this window is active for input
      ;--------------------------------------------------------------------------------------------
      
      SetActiveWindow(#Window_Location)
      
      ;--------------------------------------------------------------------------------------------
      ; Make sure the itemname field is ready for input
      ;--------------------------------------------------------------------------------------------
      
      SetActiveGadget(#Gadget_Location_Location)
      
      ;--------------------------------------------------------------------------------------------
      ; Add the message to the main window fake status area
      ;--------------------------------------------------------------------------------------------
      
      SetInfoBarArea("Headings",  "Info",  Message.s + NextRecordString.s, #PB_Compiler_Procedure)
      
      ;--------------------------------------------------------------------------------------------
      ; Go here if we could not open this data sheet window
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      SetInfoBarArea("Headings",  "Warn",  "Could not open the "  + Program\LastWindow  + " window", #PB_Compiler_Procedure)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; Go here if there is another program window open. Programmer error
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Error",  Program\LastWindow + " is still open bozo, close that one first", #PB_Compiler_Procedure)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; Nothing more to do folks
  ;------------------------------------------------------------------------------------------------
  
EndProcedure


; IDE Options = PureBasic 5.43 LTS (Windows - x86)
; CursorPosition = 111
; FirstLine = 78
; Folding = -
; EnableAsm
; EnableUnicode
; EnableXP
; HideErrorLog