﻿;==========================================================================================================================================================
; Search any records based on fuzzy Search logic
;==========================================================================================================================================================

Procedure AdvancedSearchRun()
 
  ;------------------------------------------------------------------------------------------------
  ; Clear the existing display if it contains items
  ;------------------------------------------------------------------------------------------------
  
  If CountGadgetItems(#Gadget_Keeper_Titles) <> #NoItems
    
    ClearGadgetItems(#Gadget_Keeper_Titles)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; Get the type of SQL Search to perform
  ;------------------------------------------------------------------------------------------------
  
  SearchType.s    = GetGadgetText(#Gadget_Search_SearchType)
  
  ;------------------------------------------------------------------------------------------------
  ; Get the field to perform the Search in
  ;------------------------------------------------------------------------------------------------
  
  SearchField.i   = GetGadgetState(#Gadget_Search_SearchField)
  
  ;------------------------------------------------------------------------------------------------
  ; Get the Search text for recipes to return
  ;------------------------------------------------------------------------------------------------
  
  SearchText.s    = RepQuote(GetGadgetText(#Gadget_Search_SearchText))
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  Program\LastSearch  = SearchText.s
  
  QueryString.s       = "UPDATE Preferences "
  
  QueryString.s       + "SET LastSearch = '" + RepQuote(Program\LastSearch)  + "') "
  
  QueryString.s       + "WHERE Record = '1'"
  
  DatabaseUpdate(Program\DatabaseHandle, QueryString.s)
  
  ;------------------------------------------------------------------------------------------------
  ; Get the field to perform the sort on
  ;------------------------------------------------------------------------------------------------

  SortField.i   = GetGadgetState(#Gadget_Search_SortField)
  
  ;------------------------------------------------------------------------------------------------
  ; Get the SQL sort direction
  ;------------------------------------------------------------------------------------------------

  If GetGadgetState(#Gadget_Search_oSortAcending)
    
    SortDirection.s = " ASC "
    
  ElseIf GetGadgetState(#Gadget_Search_oSortDescending)
    
    SortDirection.s = " DESC "
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; Get the limit for the number of data items to return
  ;------------------------------------------------------------------------------------------------
  
  If GetGadgetState(#Gadget_Search_cbQueryReturnLimit) = 1
    
    SortLimit.s = " LIMIT " + GetGadgetText(#Gadget_Search_sQueryReturnLimit)
    
  Else
    
    SortLimit.s = " LIMIT 500"
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; Alias the Search field numbers to database field names
  ;------------------------------------------------------------------------------------------------
  
  Select SearchField.i

    Case   0  : FieldText.s = "All fields"

    Case   1  : FieldText.s = "Title"

    Case   2  : FieldText.s = "Information"

    Case   3  : FieldText.s = "Category"

    Case   4  : FieldText.s = "Favourite"

    Case   5  : FieldText.s = "Locked"

    Case   6  : FieldText.s = "Deleted"

    Case   7  : FieldText.s = "Updated"
      
    Case   8  : FieldText.s = "Owner"

    Case   9  : FieldText.s = "Attachments"

    Case  10  : FieldText.s = "Record"

  EndSelect
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  Select SortField.i

    Case   0  : FieldText.s = "All fields"

    Case   1  : FieldText.s = "Title"

    Case   2  : FieldText.s = "Information"

    Case   3  : FieldText.s = "Category"

    Case   4  : FieldText.s = "Favourite"

    Case   5  : FieldText.s = "Locked"

    Case   6  : FieldText.s = "Deleted"

    Case   7  : FieldText.s = "Updated"

    Case   8  : FieldText.s = "Owner"

    Case   9  : FieldText.s = "Attachments"

    Case  10  : FieldText.s = "Record"

  EndSelect
  
  ;------------------------------------------------------------------------------------------------
  ; Set the initial total item counter
  ;------------------------------------------------------------------------------------------------
  
  ItemCounter.i   = #NoItems
  
  ;------------------------------------------------------------------------------------------------
  ; Alias the Search type text to SQLite operators
  ;------------------------------------------------------------------------------------------------
  
  SearchCommand.s = SetSearchQuery(SearchType.s, SearchText.s)
  
  ;------------------------------------------------------------------------------------------------
  ; Search all fields or just a specific field
  ;------------------------------------------------------------------------------------------------
  
  Select FieldText.s
      
    Case "All fields"
      
      QueryString.s = "SELECT * FROM Keeper WHERE ("
      
      QueryString.s + "Title"            + SearchCommand.s
      
      QueryString.s + " OR Information"  + SearchCommand.s
      
      QueryString.s + " OR Category"     + SearchCommand.s
      
      QueryString.s + " OR Favourite"    + SearchCommand.s
      
      QueryString.s + " OR Locked"       + SearchCommand.s
      
      QueryString.s + " OR Deleted"      + SearchCommand.s
      
      QueryString.s + " OR Updated"      + SearchCommand.s
      
      QueryString.s + " OR Owner"        + SearchCommand.s
      
      QueryString.s + " OR Attachments"  + SearchCommand.s
      
      QueryString.s + " OR Record"       + SearchCommand.s
      
      QueryString.s + ")"
      
      QueryString.s + " ORDER BY " + SortFieldText.s + SortDirection.s + SortLimit.s
      
      ;--------------------------------------------------------------------------------------------
      ; Category & Owner can have multiple items searched separated by commas
      ;--------------------------------------------------------------------------------------------
      
    Case "Information", "Category", "Owner"
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If GetGadgetState(#Gadget_Search_oCheckOr) = #True
        
        SearchMode.s = " OR "
        
      ElseIf  GetGadgetState (#Gadget_Search_oCheckAnd) = #True
        
        SearchMode.s = " AND "
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Search out how many items that the user wishes to search for
      ;--------------------------------------------------------------------------------------------
      
      NumberOfItems.i = CountString(SearchText.s, ",")
      
      ;--------------------------------------------------------------------------------------------
      ; Separate each item into a separate SQL component with the right keyword
      ;--------------------------------------------------------------------------------------------
      
      If NumberOfItems.i
        
        For GetItems.i = 1 To NumberOfItems.i + 1
          
          GetItemString.s + FieldText.s
          
          GetItemString.s + SetSearchQuery(SearchType.s, StringField(SearchText.s, GetItems.i, ","))
          
          If GetItems.i <> NumberOfItems.i + 1
            
            GetItemString.s + SearchMode.s
            
          Else
            
            GetItemString.s + #EmptyString
            
          EndIf
          
        Next GetItems.i
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        QueryString.s = "SELECT * FROM Keeper WHERE ("
        
        QueryString.s + GetItemString.s
        
        QueryString.s + ")"
        
        QueryString.s + " ORDER BY " + SortFieldText.s + SortDirection.s + SortLimit.s
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------

      Else
        
        QueryString.s = "SELECT * FROM Keeper WHERE ("
        
        QueryString.s + FieldText.s
        
        QueryString.s + SetSearchQuery(SearchType.s, SearchText.s)
        
        QueryString.s + ")"
        
        QueryString.s + " ORDER BY " + SortFieldText.s + SortDirection.s + SortLimit.s
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------

    Default
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      QueryString.s = "SELECT * FROM Keeper WHERE "
      
      QueryString.s + FieldText.s
      
      QueryString.s + SearchCommand.s
      
      QueryString.s + " ORDER BY " + SortFieldText.s + SortDirection.s + SortLimit.s

      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
  EndSelect
  
  ;------------------------------------------------------------------------------------------------
  ; Run the query with the chosen details
  ;------------------------------------------------------------------------------------------------
  
  If DatabaseQuery(Program\DatabaseHandle,  QueryString.s)  <> #DatabaseQueryFail
    
    While NextDatabaseRow(Program\DatabaseHandle)
      
      Keeper\Title        =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 0))
      
      Keeper\Information  =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 1))
      
      Keeper\Category     =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 2))
      
      Keeper\Favourite    =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 3)) 
      
      Keeper\Locked       =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 4)) 
      
      Keeper\Deleted      =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 5))
      
      Keeper\Updated      =    IsoDate(GetDatabaseString(Program\DatabaseHandle, 6), "NormalDate")
      
      Keeper\Owner        =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 7))
      
      Keeper\Attachments  =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 8))

      Keeper\Record       =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 9))
      
      ;--------------------------------------------------------------------------------------------
      ; Set the correct icon in the ListIcongadget for normal or favourite recipe
      ;--------------------------------------------------------------------------------------------
      
      DisplayTitleString(ItemCounter.i, "AddMode")
      
      ItemCounter.i + 1
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Wend
    
    ;----------------------------------------------------------------------------------------------
    ; Clear the query buffer
    ;----------------------------------------------------------------------------------------------
    
    FinishDatabaseQuery(Program\DatabaseHandle)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    SetInfoBarArea("Records", "", Str(ItemCounter.i), "SearchDataExtendedNow")
    
    ;----------------------------------------------------------------------------------------------
    ; Load the items for the first location in the list
    ;----------------------------------------------------------------------------------------------
    
    If ItemCounter.i <> #NoItems
      
      SetActiveGadget(#Gadget_Keeper_Titles)
      
      SetGadgetItemState(#Gadget_Keeper_Titles, 0, #PB_ListIcon_Selected)
      
      PostEvent(#PB_Event_Gadget, #Window_Keeper, #Gadget_Keeper_Titles, #PB_EventType_Change)
      
    Else
      
      ;--------------------------------------------------------------------------------------------
      ; Clear all data on main form before putting in the new details
      ;--------------------------------------------------------------------------------------------
      
      SetGadgetText(#Gadget_Keeper_Details,    #EmptyString)
      
      SetGadgetText(#Gadget_Keeper_sCategory,  #EmptyString)
      
      SetGadgetText(#Gadget_Keeper_sOwner,     #EmptyString)
      
      SetGadgetText(#Gadget_Keeper_sUpdated,   #EmptyString)
      
      SetGadgetText(#Gadget_Keeper_sStatus,    #EmptyString)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    EndIf
    
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    SetInfoBarArea("Records", #EmptyString, Str(CountGadgetItems(#Gadget_Keeper_Titles)), "SearchDataExtendedNow")
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings", "Error", "The database query failed. " + DatabaseError(), "SearchDataExtendedNow")
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  If GetGadgetState(#Gadget_Search_cbCheckEnter)
    
    CloseMyWindow(#Window_Search)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure

; IDE Options = PureBasic 5.43 LTS (Windows - x86)
; CursorPosition = 4
; Folding = -
; EnableAsm
; EnableUnicode
; EnableXP