﻿;==============================================================================================================================================================================
; 
;==============================================================================================================================================================================

; Super Printer - DPI independant print by Rui Carvalho - 2003
;
; Part of GIA - Lift Management Software
; Freeware
;
; mailto: rui-carvalho@bigfoot.com
;
; Original printer code by Fred
;
; Modified by Reginald Venaglia
;
; Adapted by Fangbeast

;==============================================================================================================================================================================  
; Print the items list
;==============================================================================================================================================================================

Procedure PrintKeyboardShortCuts()
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  Macro Pageheading()
    
    PageHeading.s = "Recipes manager keyboard shortcuts list"
    
    DrawingFont(FontID(HeadingFontId.i))
    
    FrontColor(RGB(255, 0, 0))
    
    DrawText(xcm(0.35 * 2.0), ycm(1.0), PageHeading.s)
    
  EndMacro
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  Macro PageNumber(CurrentPage)
    
    PageNumberHeading.s = "Page (" + Str(CurrentPage)  + ")"

    DrawingFont(FontID(PageNumberFontId))

    FrontColor(RGB(211, 211, 211))

    DrawText(xcm(9.2  * 2.0), ycm(1.0), PageNumberHeading.s)                                   ; DrawText(xcm(8.0  * 2.0), ycm(1.0), PageNumberHeading.s)
    
  EndMacro
  
  ;------------------------------------------------------------------------------------------------
  ; Of course, this means the font isn't proportional and I can't seem to space across columns
  ;------------------------------------------------------------------------------------------------
  
  HeadingFontId.i     =  LoadFont(#PB_Any, "Arial", ycm(0.4), #PB_Font_Bold | #PB_Font_HighQuality)                         ; For page heading
  
  SectionFontId.i     =  LoadFont(#PB_Any, "Arial", ycm(0.4), #PB_Font_Bold | #PB_Font_Underline | #PB_Font_HighQuality)    ; For keyboard shortcut sections
  
  NormalFontId.i      =  LoadFont(#PB_Any, "Arial", ycm(0.4), #PB_Font_HighQuality)                                         ; For keyboard shortcut text
  
  PageNumberFontId.i  =  LoadFont(#PB_Any, "Arial", ycm(0.3), #PB_Font_HighQuality)                                         ; For page numbers
  
  ;------------------------------------------------------------------------------------------------
  ; Note! LinePosition.d must be a double for line position
  ;------------------------------------------------------------------------------------------------
  
  Protected LinePosition.d, PageNumberHeading.s, PageNumber.i, PageLength.i, PrintingWidth.i
  
  Protected PageHeading.s, NumberOfLines.i, CurrentLine.i
  
  ;------------------------------------------------------------------------------------------------
  ; Get number items/lines in ListIcon or EditorGadget
  ;------------------------------------------------------------------------------------------------
  
  NumberOfLines.i = CountGadgetItems(#Gadget_Shortcuts_Shortcuts)
  
  ;------------------------------------------------------------------------------------------------
  ; Set up the printed page length
  ;------------------------------------------------------------------------------------------------
  
  PageLength.i  = 27.0  ; A4 paper portrait leaving some white space on bottom
  
  ;------------------------------------------------------------------------------------------------
  ; Lineposition 0.35 is start point lefthand side paper ycm(1.0) is start line position from top of paper
  ; Each subsequent line thereafter add 0.5 including spaced lines. Now start print routine.
  ;------------------------------------------------------------------------------------------------
  
  If PrintRequester()
    
    ;----------------------------------------------------------------------------------------------
    ; Get the page width of the printer
    ;----------------------------------------------------------------------------------------------
    
    PrintingWidth.i = PrinterPageWidth()
    
    ;----------------------------------------------------------------------------------------------
    ; Set first page number here
    ;----------------------------------------------------------------------------------------------
    
    PageNumber.i        = 1
    
    ;----------------------------------------------------------------------------------------------
    ; Give a job name for printer to use
    ;----------------------------------------------------------------------------------------------
    
    If StartPrinting("Keyboard Shortcuts List")
      
      ;--------------------------------------------------------------------------------------------
      ; Tell PB what the print output is to be sent to
      ;--------------------------------------------------------------------------------------------
      
      If StartDrawing(PrinterOutput())
        
        ;------------------------------------------------------------------------------------------
        ; Set the drawing mode for the printed surface
        ;------------------------------------------------------------------------------------------
        
        DrawingMode(#PB_2DDrawing_Transparent)
        
        ;------------------------------------------------------------------------------------------
        ; Set font & colour for any main page heading and draw it
        ;------------------------------------------------------------------------------------------
        
        Pageheading()
        
        ;------------------------------------------------------------------------------------------
        ; Draw the page number heading and it's header text on the right in black
        ;------------------------------------------------------------------------------------------
        
        PageNumber(PageNumber)
        
        ;------------------------------------------------------------------------------------------
        ; Place line under headings (Will be a light grey)
        ;------------------------------------------------------------------------------------------
        
        FrontColor(RGB(164, 164, 164))
        
        Line(xcm(0.35), ycm(1.7),  PrintingWidth.i - 170, 1)                                         ; Note for straight line last parameter is 1
        
        ;------------------------------------------------------------------------------------------
        ; Set start line number position for start items list
        ;------------------------------------------------------------------------------------------
        
        LinePosition = 2.5
        
        ;------------------------------------------------------------------------------------------
        ; Get a line from ListIcon or editorGadget or table or array to print into string.s
        ;------------------------------------------------------------------------------------------
        
        For CurrentLine.i = 0 To  NumberOfLines.i - 1                                               ; PB gadgets are 0 based
          
          Explanation.s       = GetGadgetItemText(#Gadget_Shortcuts_Shortcuts, CurrentLine.i, 0)    ; Get the shortcut key explanation or heading
          
          KeyboardShortcut.s  = GetGadgetItemText(#Gadget_Shortcuts_Shortcuts, CurrentLine.i, 1)    ; Get the keyboard shortcut text
          
          ; Set foreground and background colours                                                   ; Bugger, bugger, bum!
          
          If Explanation.s  = #EmptyString And KeyboardShortcut.s  = #EmptyString                  ; This is an empty line
            
            DrawingFont(FontID(NormalFontId.i))                                                     ; Normal text
            
            FrontColor(RGB(0, 0, 0))                                                                ; Should be paper white
            
            DrawText(xcm(0.35 * 2.0), ycm(LinePosition), Explanation.s)                             ; Draw a blank line
            
          ElseIf Explanation.s <> #EmptyString And KeyboardShortcut.s = #EmptyString               ; This is a heading
            
            LinePosition + 0.6
            
            DrawingFont(FontID(SectionFontId.i))                                                    ; Bold and underline
            
            FrontColor(RGB(32, 52, 230))                                                            ; Should be black
            
            DrawText(xcm(0.35 * 2.0), ycm(LinePosition), UCase(Explanation.s))                      ; Draw the heading string
            
          ElseIf Explanation.s <> #EmptyString And KeyboardShortcut.s <> #EmptyString              ; This is a keyboard shortcut line
            
            DrawingFont(FontID(NormalFontId.i))                                                     ; Normal text
            
            FrontColor(RGB(0, 0, 0))                                                                ; Should be black
            
            DrawText(xcm(0.35 * 2.0), ycm(LinePosition), Explanation.s)                             ; 
            
            DrawText(xcm(8.0  * 2.0), ycm(LinePosition), KeyboardShortcut.s)                        ; 
            
          EndIf                                                                                    ;
          
          ;----------------------------------------------------------------------------------------
          ; Draw 2 space lines
          ;----------------------------------------------------------------------------------------
          
          LinePosition + 0.6
          
          ;----------------------------------------------------------------------------------------
          ; The next section only required if list could not fit on a single page
          ;----------------------------------------------------------------------------------------
          
          If LinePosition >= PageLength.i
            
            NewPrinterPage()
            
            ; Increment page PageNumber
            
            PageNumber.i + 1
            
            ;--------------------------------------------------------------------------------------
            ; Set font & colour for any main page heading and draw it
            ;--------------------------------------------------------------------------------------
            
            Pageheading()                                                                           ; Macro to draw page heading
            
            ;--------------------------------------------------------------------------------------
            ; Draw the page number heading and it's header text on the right in black
            ;--------------------------------------------------------------------------------------
            
            PageNumber(PageNumber)
            
            ;--------------------------------------------------------------------------------------
            ; Place line under headings
            ;--------------------------------------------------------------------------------------
            
            FrontColor(RGB(164, 164, 164))
            
            Line(xcm(0.35), ycm(1.7),  PrintingWidth.i - 170, 1)                                         ; Note for straight line last parameter is 1
            
            ;--------------------------------------------------------------------------------------
            ; Set start line number position for start next item
            ;--------------------------------------------------------------------------------------
            
            LinePosition = 2.5
            
            ;--------------------------------------------------------------------------------------
            ; 
            ;--------------------------------------------------------------------------------------
            
          EndIf
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
        Next CurrentLine.i
        
        ;------------------------------------------------------------------------------------------
        ; Don't ever forget this
        ;------------------------------------------------------------------------------------------
        
        StopDrawing()
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      StopPrinting()
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  FreeFont(HeadingFontId.i)
  
  FreeFont(SectionFontId.i)
  
  FreeFont(NormalFontId.i )
  
  FreeFont(PageNumberFontId.i)
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure
; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; CursorPosition = 17
; Folding = -
; EnableAsm
; EnableUnicode
; EnableXP