﻿;==============================================================================================================================================================================
; TI-994A's routine for vector printing
;==============================================================================================================================================================================

Procedure TI_PrintRecipe(recipeName.s, image.i, ingredients.s, method.s)
 
  recipeTitleFont   = LoadFont(#PB_Any, "Arial",            24, #PB_Font_Bold)
  
  recipeBodyFonts   = LoadFont(#PB_Any, "Times New Roman",  14)
  
  recipeHeaderFonts = LoadFont(#PB_Any, "Times New Roman",  22, #PB_Font_Bold | #PB_Font_Italic)
 
  If PrintRequester()
    
    If StartPrinting("Recipes: " + recipeName)
      
      If StartVectorDrawing(PrinterVectorOutput(#PB_Unit_Millimeter))
       
        frameWidth  = 1
        
        singleSpace = 5
        
        doubleSpace = 10
        
        pageMargin  = 10
        
        pageWidth   = VectorOutputWidth()
        
        pageHeight  = VectorOutputHeight()
       
        MovePathCursor(pageMargin, pageMargin)
        
        AddPathLine(pageWidth - (pageMargin * 2), 0,      #PB_Path_Relative)
        
        AddPathLine(0, pageHeight - (pageMargin * 2),     #PB_Path_Relative)
        
        AddPathLine(- (pageWidth - (pageMargin  * 2)), 0, #PB_Path_Relative)
        
        AddPathLine(0, -(pageHeight - (pageMargin * 2)),  #PB_Path_Relative)
        
        StrokePath(frameWidth, #PB_Path_SquareEnd)
       
        VectorFont(FontID(recipeTitleFont))
        
        VectorSourceColor(RGBA(0, 0, 155, 255))
       
        titleFontSize = 24 / 2.83
        
        titleXMargin = (pageWidth - VectorTextWidth(recipeName)) / 2
        
        While titleXMargin < pageMargin
          
          If titleFontSize > 1
            
            titleFontSize - 1
            
            VectorFont(FontID(recipeTitleFont), titleFontSize)
            
            titleXMargin = (pageWidth - VectorTextWidth(recipeName)) / 2
            
          Else
            
            Break
            
          EndIf
          
        Wend
        
        titleYMargin = pageMargin + singleSpace
       
        MovePathCursor(titleXMargin, titleYMargin)
        
        DrawVectorText(recipeName)
       
        startX = pageMargin + singleSpace
        
        startY = PathCursorY() + (singleSpace * 3)
        
        MovePathCursor(startX, startY)
       
        If IsImage(image)
          
          If ImageWidth(image) <= ImageHeight(image)
            
            imgAspect.d = ImageWidth(image) / ImageHeight(image)
            
            imgHeight.d  = (pageWidth - (pageMargin * 2)) / 2
            
            imgWidth.d = imgHeight * imgAspect
            
          Else
            
            imgAspect.d = ImageHeight(image) / ImageWidth(image)
            
            imgWidth.d  = (pageWidth - (pageMargin * 2)) / 2
            
            imgHeight.d = imgWidth * imgAspect
            
          EndIf
         
          DrawVectorImage(ImageID(image), 255, imgWidth, imgHeight)
          
          startX = PathCursorX() + singleSpace
          
        EndIf
       
        imageBase = PathCursorY()
       
        VectorFont(FontID(recipeHeaderFonts))
        
        VectorSourceColor(RGBA(0, 0, 0, 255))
        
        MovePathCursor(startX, startY)
        
        DrawVectorText("Ingredients")
       
        startY = PathCursorY() + doubleSpace
        
        VectorFont(FontID(recipeBodyFonts))
        
        MovePathCursor(startX, startY)
        
        paragraphWidth = pageWidth - (startX + pageMargin + singleSpace)
        
        paragraphHeight = pageHeight - (startY + pageMargin)
        
        DrawVectorParagraph(ingredients, paragraphWidth, paragraphHeight)
       
        If IsImage(image)
          
          If PathCursorY() > imageBase
            
            indentY = PathCursorY() + doubleSpace
            
            indentWidth = (pageWidth - ((pageMargin * 2) + (singleSpace * 3))) - paragraphWidth
            
          EndIf
          
          startY = imageBase
          
        Else
          
          startY = PathCursorY()
          
        EndIf

        startY + doubleSpace
        
        startX = pageMargin + singleSpace
        
        VectorFont(FontID(recipeHeaderFonts))
        
        VectorSourceColor(RGBA(0, 0, 0, 255))
        
        MovePathCursor(startX, startY)
        
        DrawVectorText("Method")
       
        startY = PathCursorY() + doubleSpace
        
        VectorFont(FontID(recipeBodyFonts))
        
        MovePathCursor(startX, startY)
       
        If indentY
          
          paragraphWidth = indentWidth
          
          paragraphHeight = indentY - (startY - doubleSpace)
          
          requiredHeight = 0
         
          While requiredHeight < paragraphHeight
            
            fragmentLen + 1
            
            If fragmentLen > Len(method)
              
              Break
              
            EndIf           
            
            partialParagraph$ = Mid(method, 1, fragmentLen)
            
            requiredHeight = VectorParagraphHeight(partialParagraph$, paragraphWidth, paragraphHeight)
            
          Wend
          
          If fragmentLen < Len(method)
            
            While Mid(partialParagraph$, fragmentLen, 1) <> " "
              
              fragmentLen - 1
              
              If fragmentLen < 1
                
                Break
                
              EndIf
              
            Wend 
            
          EndIf
         
          partialParagraph$ = Mid(partialParagraph$, 1, fragmentLen)
          
          DrawVectorParagraph(partialParagraph$, paragraphWidth, paragraphHeight)
         
          method = Mid(method, fragmentLen + 1)
          
          startY = PathCursorY()
          
        EndIf
       
        If Trim(method) <> ""
          
          While Trim(method) <> ""
           
            paragraphWidth = pageWidth - ((pageMargin * 2) + doubleSpace)
            
            paragraphHeight = pageHeight - (startY + pageMargin)
            
            requiredHeight = 0
            
            fragmentLen = 0
           
            While requiredHeight < paragraphHeight
              
              fragmentLen + 1
              
              If fragmentLen > Len(method)
                
                Break
                
              EndIf
              
              partialParagraph$ = Mid(method, 1, fragmentLen)
              
              requiredHeight = VectorParagraphHeight(partialParagraph$, paragraphWidth, paragraphHeight)
              
            Wend
           
            If fragmentLen < Len(method)
              
              While Mid(partialParagraph$, fragmentLen, 1) <> " "
                
                fragmentLen - 1
                
                If fragmentLen < 1
                  
                  Break
                  
                EndIf
                
              Wend 
              
            EndIf
           
            partialParagraph$ = Mid(partialParagraph$, 1, fragmentLen)
            
            DrawVectorParagraph(partialParagraph$, paragraphWidth, paragraphHeight)
           
            method = Mid(method, fragmentLen + 1)
           
            If Trim(method) = ""
              
              Break
              
            EndIf
         
            NewVectorPage()
           
            MovePathCursor(pageMargin, pageMargin)
            
            AddPathLine(pageWidth - (pageMargin * 2), 0,      #PB_Path_Relative)
            
            AddPathLine(0, pageHeight - (pageMargin * 2),     #PB_Path_Relative)
            
            AddPathLine(- (pageWidth - (pageMargin  * 2)), 0, #PB_Path_Relative)
            
            AddPathLine(0, -(pageHeight - (pageMargin * 2)),  #PB_Path_Relative)
            
            StrokePath(frameWidth, #PB_Path_SquareEnd)
           
            VectorFont(FontID(recipeTitleFont), titleFontSize)
            
            VectorSourceColor(RGBA(0, 0, 155, 255))
           
            MovePathCursor(titleXMargin, titleYMargin)
            
            DrawVectorText(recipeName)           

            startY = PathCursorY() + (singleSpace * 3)   
            
            VectorFont(FontID(recipeBodyFonts))
            
            VectorSourceColor(RGBA(0, 0, 0, 255))
            
            MovePathCursor(startX, startY)
           
          Wend   
          
        EndIf
       
        StopVectorDrawing()
       
      EndIf
      
    EndIf
    
    StopPrinting()
    
  EndIf
 
EndProcedure 


; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; CursorPosition = 298
; FirstLine = 271
; Folding = -
; EnableAsm
; EnableUnicode
; EnableXP